# -*- coding: iso-8859-1 -*-

# Mattricks, a free helper program for hattricks players.
# Copyright (C) 2003-2004 Mattricks Team
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""Common functions for getting various kinds of data."""

from Common import *

import Countries

def add_currency(amount_of_money, always_sign = False):
    """Create a currency string from an amount of money
    using current teams league currency."""

    lid = get_country_league_id()
    (htname, name, currency, is_prefix, exchangerate, flag) = Countries.country_info[lid]
    amount = int(float(amount_of_money) / float(exchangerate))
    return add_currency_to_money(amount, currency, is_prefix, always_sign)

import Database
import XMLParse

def get_league_season(league_id):
    """Returns the current season of a league.

     - league_id -- The league id of the league to get season from."""

    data = XMLParse.get(["world details"], ["LeagueList.League.xml_list",
                                            "LeagueList.League.xml_attrs.Index"])

    lindeces = data.get("LeagueList.League.xml_list")
    if lindeces is None:
        lindeces = data.get("LeagueList.League.xml_attrs.Index")
        if lindeces is None:
            return
        else:
            lindeces = [str(lindeces)]
    else:
        lindeces = ["+" + x for x in lindeces.split(",")]

    paths = []
    for index in lindeces:
        paths.append("LeagueList.League%s.LeagueID" % index)
        paths.append("LeagueList.League%s.Season" % index)

    data = XMLParse.get(["world details"], paths)

    for index in lindeces:
        if league_id == data.get("LeagueList.League%s.LeagueID" % index):
            return data.get("LeagueList.League%s.Season" % index)

    return None

def get_current_teamid(ret_userid = False):
    """Returns the current teamid or userid.

    If necessary, downloads a new team details xml to determine
    teamid and userid.

     - ret_userid -- if True, returns the userid instead of the teamid.
    """

    data = Database.databases.gets(None, False, ["Mattricks.current_teamid",
                                                 "Mattricks.current_userid"])
    teamid = data.get("Mattricks.current_teamid")
    userid = data.get("Mattricks.current_userid")

    if teamid is None or userid is None or len(teamid) == 0 or len(userid) == 0:
        data = XMLParse.get(["team details", -1], ["Team.TeamID", "User.UserID"])

        teamid = data.get("Team.TeamID")
        userid = data.get("User.UserID")

        if teamid is not None and userid is not None:
            Database.databases.sets(None, False, {"Mattricks.current_teamid": str(teamid),
                                                  "Mattricks.current_userid": str(userid)})
    else:
        teamid = int(teamid)
        userid = int(userid)

    if ret_userid:
        return userid
    return teamid

def is_supporter():
    """Returns true if the current user is a supporter"""
    data = Database.databases.gets(None, False, ["Mattricks.current_is_supporter"])
    supporter = data.get("Mattricks.current_is_supporter")

    if supporter is None or len(supporter) == 0:
        supporter = XMLParse.get(["team details", -1], ["User.HasSupporter"])
        if supporter is not None:
            Database.databases.sets(None, False, {"Mattricks.current_is_supporter": str(supporter)})
    else:
        supporter = string2bool(supporter)

    return supporter

country_league_id_cache = {}

def get_country_league_id(teamid = None):
    """Returns the league id of the country for the given team.

    This function caches the retrieved info for best speed.

     - teamid -- the teamid to get country info for,
                 if None uses current team."""

    global country_league_id_cache

    if country_league_id_cache.has_key(teamid):
        return country_league_id_cache[teamid]

    if teamid is None:
        tid = get_current_teamid()
    else:
        tid = teamid

    data = XMLParse.get(["team details", tid], ["Team.League.LeagueID"])
    if data is not None:
        country_league_id_cache[teamid] = data
    return data

def get_xml_list_indices(path, data, xml_attr = "Index"):
    """Get all indices of a particular path.

     - path -- The path to the list of the indeces.
     - data -- The data to get from.
     - xml_attr -- The index form in the data to use."""

    indices = data.get(path + ".xml_list", "").split(",")
    if len(indices) == 1 and indices[0] == "":
        m = data.get(path + ".xml_attrs." + xml_attr)
        if m is None:
            indices = []
    else:
        indices = ["+" + x for x in indices]

    return indices

class DataStore:
    """Conevnience class for handling data retrieved by XMLParse.get()."""

    comparison_key = None

    def __init__(self, data, single = False):
        """Takes data and initializes the class.

         - data -- the data dictionary.
         - single -- true if the data does not containn both display
                     and convert values."""

        self.data = data
        self.__single = single

    def get(self, key, def_val = None, display = False):
        """Return the value for the specified key.

        For single, the display argument is ignored.

         - key -- the specifed key.
         - def_val -- what to return if the key wasn't found.
         - display -- if true, return the display converted value."""

        d = self.data.get(key)

        if d is None:
            return def_val

        if self.__single:
            return d

        if display:
            return d[1]
        return d[0]

    def set_comparison_key(key):
        """Set the key to use as comparison key."""
        DataStore.comparison_key = key

    set_comparison_key = staticmethod(set_comparison_key)

    def __ne__(self, y):
        """Not equal with. x.__ne__(y) <=> x != y"""

        if self.comparison_key is None:
            return False

        return self.get(self.comparison_key, None) != y.get(self.comparison_key, None)

    def __eq__(self, y):
        """Equal with. x.__eq__(y) <=> x == y"""

        if self.comparison_key is None:
            return False

        return self.get(self.comparison_key, None) == y.get(self.comparison_key, None)

    def __le__(self, y):
        """Less than or equal. x.__le__(y) <=> x <= y"""

        if self.comparison_key is None:
            return False

        return self.get(self.comparison_key, None) <= y.get(self.comparison_key, None)

    def __lt__(self, y):
        """Less than. x.__le__(y) <=> x < y"""

        if self.comparison_key is None:
            return False

        return self.get(self.comparison_key, None) < y.get(self.comparison_key, None)

    def __ge__(self, y):
        """Greater than or equal. x.__le__(y) <=> x >= y"""

        if self.comparison_key is None:
            return False

        return self.get(self.comparison_key, None) >= y.get(self.comparison_key, None)

    def __gt__(self, y):
        """Greater than. x.__le__(y) <=> x > y"""

        if self.comparison_key is None:
            return False

        return self.get(self.comparison_key, None) > y.get(self.comparison_key, None)

    def __len__(self):
        """The amount of data in this storage."""

        return len(self.data)

    def __str__(self):
        """Make a string of this storage."""

        return str(self.data)

    def __repr__(self):
        """Make a string of this storage."""

        return repr(self.data)
